#!/usr/bin/perl 
##########################################################################
#                                                                        #
# Perl Script for Resend lost Accouting packet to portaone Repius Server #
#                                                                        #
# Modified on 110324 by yannick.vaucher@switzernet.com                   #
#                                                                        #
# Switzernet(c)2011                                                      #
#                                                                        #
##########################################################################
# my modules
use strict;
use warnings;
use POSIX;
use Config::IniFiles;
use Switch;
use DBI;
##########################################################################
# Setting
my $lock_file = '/var/run/exec-replication.pid';
my $runas_user = 'root';
my $config_file = '/etc/dba/setup-mysql-replication.conf';
##########################################################################
# Globals
my ($dbh1,$dbh2,$User,$Pass,$master_host,$master_user);
##########################################################################
#                             Initialization                             #
##########################################################################
# Check if already running
if( -e $lock_file ) {
	open(PID,$lock_file);
	my $pid=<PID>;
	close PID;
	chomp $pid;
	if( !-e "/proc/$pid" ) {
		print STDERR "Lock file present, but no process with pid=$pid.\n";
		die "Can't delete lock file $lock_file\n" if !unlink $lock_file;
		print STDERR "Lock file has been removed.\n";
	} else {
		die "Lockfile present, another copy is punning pid=$pid\n";
	}
}
my ($name, $passwd, $uid, $gid) = getpwnam($runas_user) or die "$runas_user not in passwd file";;
##########################################################################
# set C locale
POSIX::setlocale(LC_ALL, "C");
##########################################################################
# Become daemon
my $pid;
if( !defined($pid = fork()) ) {
    die "cannot fork: $!";
} elsif ($pid) {
    # Create lockfile, and finish parent process
    open(PID, "> $lock_file") || die "exec-replication.pl: Unable to create lockfile $lock_file\n";
    print PID "$pid";
    close PID;
    chown $uid, $gid, $lock_file;
    exit;
} else {
    # daemon
    setpgrp();
    select(STDERR); $| = 1;
    select(STDOUT); $| = 1;
}

##########################################################################
# Install signall handler
$SIG{INT} = \&safe_exit;
$SIG{QUIT} = \&safe_exit;
$SIG{TERM} = \&safe_exit;
$SIG{HUP} = \&load_config;
##########################################################################
# Drop privileges
setuid($uid);
$< = $uid;
$> = $uid;
##########################################################################
# Signal Handlers
sub safe_exit {
    unlink $lock_file ;
    exit;
}
##########################################################################
#                                FUNCTION                                #
##########################################################################
# Load config from mysql.conf
sub load_config {
    my $conf=Config::IniFiles->new(-file => $config_file);
    return 0 if !defined $conf ;
    $User  = $conf->val('SLAVE','User');
    $Pass  = $conf->val('SLAVE','Pass');      
    return 1;
}
##########################################################################
# Astrad Radius DB connect
sub db_connect {
my $dbh = DBI->connect_cached("DBI:mysql:".$_[0].";host=".$_[1],$_[2],$_[3]);
return $dbh if $dbh ;
safe_exit();
}
##########################################################################
# 
sub exec_request {
my $request="delete from request where id in (";
my $sth = $dbh1->prepare("select id,val from request order by id limit 1000");
safe_exit() if !$sth->execute();
return 0 if  !$sth->rows;
print $sth->rows."\n";
while (my $result = $sth->fetchrow_hashref) {
    $request.=$result->{id}."," if $dbh2->do($result->{val});
    }
$sth->finish;
$request=substr($request,0,length($request)-1).")";
$dbh1->do($request);
return 1;
}
##########################################################################
#                                 main()                                 #
##########################################################################
load_config();
for (my $i=0;$i<30;$i++) {
  $dbh1=db_connect("astrad","localhost",$User,$Pass);
  $dbh2=db_connect("asterisk","localhost",$User,$Pass);
  $i=0 if exec_request();
  sleep 1 if $i;
}
safe_exit();
##########################################################################
#                                  END                                   #
##########################################################################
